#!/usr/bin/env python3
# -*- coding:utf-8 -*-⏎

from __future__ import print_function
import sys

if sys.version_info<(3,0,0):
    reload(sys)
    sys.setdefaultencoding('utf8')


import urllib.request
import urllib.parse
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|
def read_url(url, retry=1, encoding="utf-8", verbose=1):
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|
    if not url:
        if verbose >= 1:
            print("no url")
        return False

    for _ in range(0,retry):
        try:
            c = urllib.request.urlopen(url)
            content = c.read()
            if encoding:
                content = content.decode(encoding)
            return content

        except Exception as e:
            if verbose >= 1:
                print(e)
            continue

    return False


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|
def get_url(url, retry=1, encoding="utf-8", verbose=1):
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|
    return read_url(url, retry, encoding, verbose)


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|
def post_url(url, retry=1, data_byte=None, data_dict=None, encoding="utf-8", verbose=1):
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|
    if not url:
        if verbose >= 1:
            print("no url")
        return False

    for _ in range(0,retry):
        try:
            if data_byte:
                data = data_byte
            elif data_dict:
                data = urllib.parse.urlencode( data_dict ).encode("ascii")
            else:
                data = None

            c = urllib.request.urlopen(url=url, data=data)
            content = c.read()
            if encoding:
                content = content.decode(encoding)
            return content

        except Exception as e:
            if verbose >= 1:
                print(e)
            continue

    return False



# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|
def urlenc(q):
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|
    return urllib.parse.quote_plus(q)




#!/usr/bin/env python3
# -*- coding:utf-8 -*-

from usqlite import *

db_fn="test.db"
udb = USQLite(db_fn)

q="""CREATE TABLE tmp ( id INTEGER PRIMARY KEY, 
                        name TEXT NOT NULL,
                        height REAL,
                        age INTEGER DEFAULT 0 );"""
if udb.create_table(q):
    print("created")
else:
    print("fail")


table_list = udb.get_table_names()
print(table_list)

column_list = udb.get_column_names("tmp")
print(column_list)

udb.assert_column( "tmp", "hi", "TEXT")
column_list = udb.get_column_names("tmp")
print(column_list)



#!/usr/bin/env python3
# -*- coding:utf-8 -*-⏎

import sys
import os
import time
import datetime

def datetime2datedict(d):
    rd = dict()
    rd["year"] = d.year
    rd["month"] = d.month
    rd["day"] = d.day
    rd["hour"] = d.hour
    rd["minute"] = d.minute
    rd["second"] = d.second
    return rd

def ymd2stamp(y,m,d):
    s = "%d %d %d" % (y, m, d)
    return int(time.mktime(datetime.datetime.strptime(s, "%Y %m %d").timetuple()))

def datedict2stamp(d):
    if "hour" in d: hour = d["hour"]
    else: hour = 0
    if "minute" in d: minute = d["minute"]
    else: minute = 0
    if "second" in d: second = d["second"]
    else: second = 0

    s = "%d %d %d %d %d %d" % (d["year"], d["month"], d["day"], hour, minute, second)
    return int(time.mktime(datetime.datetime.strptime(s, "%Y %m %d %H %M %S").timetuple()))

def stamp2datedict(stamp):
    d = datetime.datetime.fromtimestamp( stamp )
    return datetime2datedict(d)

def get_now_datedict():
    now = datetime.datetime.now()
    return datetime2datedict(now)

def test():
    now_dict = get_now_datedict()
    print(now_dict)

    stamp = datedict2stamp(now_dict)
    print(stamp)

    now_dict2 = stamp2datedict(stamp)
    print(now_dict2)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~4~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~8
if __name__ == "__main__":
    test()




#!/usr/bin/env python
# -*- coding:utf-8 -*-⏎

from __future__ import print_function
from fs import *

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~4~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~8
def test():
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~4~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~8
    print( "test invlid")
    root = "a"
    print(root)
    file_list = get_file_list(root)
    if file_list:
        print(fi_list)
    else:
        print("fail")

    root = "../"
    print(root)
    finfo_list = get_file_list(root)
    if finfo_list:
        for i, finfo in enumerate(finfo_list):
            print(finfo.toJson())
            if i >= 3:
                break
    else:
        print("fail")

    print( "test recursive")
    finfo_list = get_file_list(root, recursive=True)
    if finfo_list:
        for i, finfo in enumerate(finfo_list):
            print(finfo.toJson())
            if i >= 3:
                break
    else:
        print("fail")

    print( "test recursive with ext")
    finfo_list = get_file_list(root, recursive=True, ctx={ "filter": {"ext":"py"} })
    if finfo_list:
        for i, finfo in enumerate(finfo_list):
            print(finfo.toJson())
            if i >= 3:
                break
    else:
        print("fail")


    print( "test sort")
    finfo_list = get_file_list(root, recursive=True, ctx={ "filter": {"ext":"py"}, "sort": "size", "order":"desc" })
    if finfo_list:
        for i, finfo in enumerate(finfo_list):
            #print(finfo.toJson())
            print(finfo.full_fn, finfo.size)
            #if i >= 3:
            #    break
    else:
        print("fail")
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~4~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~8
if __name__ == "__main__": test()
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~4~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~8



from PIL import Image
from PIL import ImageFile
from PIL import ExifTags
ImageFile.LOAD_TRUNCATED_IMAGES = True

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|
def save_thumb(src_fn, dst_fn, size, verbose=1):
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|
    # open
    try:
        img = Image.open(src_fn)
    except Exception as e:
        if verbose>=1:
            print(e)
        return False

    # rotate based on exif
    try:
        exif_dict = img._getexif()
        if exif_dict:
            orientation = None
            for k, v in ExifTags.TAGS.items() : 
                if v == "Orientation":
                    orientation = k
                    break
            if orientation:
                if exif_dict[orientation] == 3 : 
                    img=img.rotate(180, expand=True)
                elif exif_dict[orientation] == 6 : 
                    img=img.rotate(270, expand=True)
                elif exif_dict[orientation] == 8 : 
                    img=img.rotate(90, expand=True)
                else:
                    pass
    except Exception as e:
        if verbose>=2:
            print(e)
        pass

    # prevent alpha channel warning
    try:
        img = img.convert("RGB")
    except Exception as e:
        if verbose>=2:
            print("convert RGB fail: %s" % e)
        pass

    # save
    try:
        img.thumbnail( size, Image.ANTIALIAS )
        img.save(dst_fn, "JPEG")
    except Exception as e:
        if verbose>=1:
            print("Save fail: %s" % e)
        return False

    return True


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|
def test_save_thumb():
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|
    src_fn = "/tmp/image.jpg"
    dst_fn = "/tmp/image_thumb.jpg"
    size = (120, 100)
    if save_thumb(src_fn, dst_fn, size, verbose=1):
        print("ok")
    else:
        print("fail")



if __name__ == '__main__':
    test_save_thumb()
